import { run } from "./utils.ts";
import * as path from "https://deno.land/std@0.214.0/path/mod.ts";

export class NginxController {
	private proxy_target_domain;
	private enabled_conf_dir;

	constructor(proxy_target_domain: string, enabled_conf_dir: string) {
		this.proxy_target_domain = proxy_target_domain;
		this.enabled_conf_dir = enabled_conf_dir;
	}

	public async add_proxy(domain: string, port: number, conf_dir: string) {
		const conf_file_content = `
server {
    listen 80;
    listen [::]:80;
    server_name ${domain};
    location / {
        proxy_pass http://${this.proxy_target_domain}:${port};
    }
}
`;
		const conf_file_path = path.join(conf_dir, domain + ".conf");
		const enabled_conf_file_path = path.join(this.enabled_conf_dir, domain + ".conf");
		await Deno.writeTextFile(conf_file_path, conf_file_content);
		await run("ln", "-s", await Deno.realPath(conf_file_path), enabled_conf_file_path);
		await this.reload();
		return conf_file_path;
	}

	public async remove_proxy(domain: string, conf_dir: string) {
		const conf_file_path = path.join(conf_dir, domain + ".conf");
		const enabled_conf_file_path = path.join(this.enabled_conf_dir, domain + ".conf");
		await Deno.remove(enabled_conf_file_path, { recursive: true });
		await Deno.remove(conf_file_path, { recursive: true });
		await this.reload();
	}

	public async *read_all_in_dir(conf_dir: string) {
		for await (const { name } of Deno.readDir(conf_dir)) {
			const [domain, rest] = name.split(".conf");
			if (rest !== "") continue;
			yield domain;
		}
	}

	private async reload() {
		await run("systemctl", "restart", "nginx");
	}
}
